#!bin/sh

#####################################################################################
# This Shell script and the embeded XCast/python Script are used to                 #
# Download NMME Forecast Data for domain of your interest and                       #
# Plot Season-1 / Season-2 / Season-3 Calibrated Precip Forecsts                    #
# Written/modified by Endalkachew Bekele, NOAA/CPC/International Desks, August 2023 #
# For more details on XCast, please viisit https://xcast-lib.github.io/             #
#####################################################################################

LANG=en_US.utf8
LC_ALL=en_US.utf8
unameout="$(uname -s)"

if [ "$#" -ne 4 ]; then
    echo "***************************************************************************************************************************"
    echo " error while running gen_CCA_nmme_predicted_precip.sh"
    echo "----------------- You need to provide 4 arguments -------------------------------------------------------------------------"
    echo "***************************************************************************************************************************"
    echo
    echo "The four arguments reflect geographical extent of your area:"
    echo
    echo "western lon, eastern lon, southern lat and northern lat in degreas, respectively"
    echo
    echo "Use negative sign for longitudes in western, and latitudes in the southern hemisphere"
    echo
    echo "For example, if your area extends from 10W to 12E and from 5S to 10N .."
    echo " you may run the script as .."
    echo
    echo " bash gen_CCA_nmme_predicted_precip.sh -10 12 -5 10"
    echo
    echo
exit
fi

wlon1=$1
elon1=$2
slat1=$3
nlat1=$4


wlon2=$1
elon2=$2
slat2=$3
nlat2=$4


mn=`date +"%b"`

for ld in 1 2 3; do

if [ $ld = 1 ]; then

if [ $mn == "Jan" ]; then prd='FMA'; fi
if [ $mn == "Feb" ]; then prd='MAM'; fi
if [ $mn == "Mar" ]; then prd='AMJ'; fi
if [ $mn == "Apr" ]; then prd='MJJ'; fi
if [ $mn == "May" ]; then prd='JJA'; fi
if [ $mn == "Jun" ]; then prd='JAS'; fi
if [ $mn == "Jul" ]; then prd='ASO'; fi
if [ $mn == "Aug" ]; then prd='SON'; fi
if [ $mn == "Sep" ]; then prd='OND'; fi
if [ $mn == "Oct" ]; then prd='NDJ'; fi
if [ $mn == "Nov" ]; then prd='DJF'; fi
if [ $mn == "Dec" ]; then prd='JFM'; fi
fi

if [ $ld = 2 ]; then

if [ $mn == "Jan" ]; then prd='MAM'; fi
if [ $mn == "Feb" ]; then prd='AMJ'; fi
if [ $mn == "Mar" ]; then prd='MJJ'; fi
if [ $mn == "Apr" ]; then prd='JJA'; fi
if [ $mn == "May" ]; then prd='JAS'; fi
if [ $mn == "Jun" ]; then prd='ASO'; fi
if [ $mn == "Jul" ]; then prd='SON'; fi
if [ $mn == "Aug" ]; then prd='OND'; fi
if [ $mn == "Sep" ]; then prd='NDJ'; fi
if [ $mn == "Oct" ]; then prd='DJF'; fi
if [ $mn == "Nov" ]; then prd='JFM'; fi
if [ $mn == "Dec" ]; then prd='FMA'; fi
fi


if [ $ld = 3 ]; then

if [ $mn == "Jan" ]; then prd='AMJ'; fi
if [ $mn == "Feb" ]; then prd='MJJ'; fi
if [ $mn == "Mar" ]; then prd='JJA'; fi
if [ $mn == "Apr" ]; then prd='JAS'; fi
if [ $mn == "May" ]; then prd='ASO'; fi
if [ $mn == "Jun" ]; then prd='SON'; fi
if [ $mn == "Jul" ]; then prd='OND'; fi
if [ $mn == "Aug" ]; then prd='NDJ'; fi
if [ $mn == "Sep" ]; then prd='DJF'; fi
if [ $mn == "Oct" ]; then prd='JFM'; fi
if [ $mn == "Nov" ]; then prd='FMA'; fi
if [ $mn == "Dec" ]; then prd='MAM'; fi
fi

cat>cca_nmme_predicted_precip.py<<eofPY
#!/usr/bin/env python
# coding: utf-8

import xcast as xc 
import xarray as xr 
import cartopy.crs as ccrs 
import numpy as np
import matplotlib.pyplot as plt 

model1 = xr.open_dataset('nmme_hind_precip_ld${ld}.nc')
obs1 = xr.open_dataset('chirps_hind_precip_ld${ld}.nc')
fmodel1 = xr.open_dataset('nmme_fcst_precip_ld${ld}.nc')

model = model1.precip.expand_dims({'M':[0]})
fmodel = fmodel1.precip.expand_dims({'M':[0]})
obs = obs1.precip.expand_dims({'M':[0]})

mask_missing = model.mean('time', skipna=False).mean('M', skipna=False)
mask_missing = xr.ones_like(mask_missing).where(~np.isnan(mask_missing), other=np.nan )
model = model * mask_missing

model = model.sel(lon=slice(${wlon1}, ${elon1}), lat=slice(${slat1},${nlat1}))
fmodel = fmodel.sel(lon=slice(${wlon1}, ${elon1}), lat=slice(${slat1},${nlat1}))
obs = obs.sel(lon=slice(${wlon2}, ${elon2}), lat=slice(${slat2},${nlat2}))

#model = xc.regrid(model, obs1.lon, obs1.lat)
#fmodel = xc.regrid(fmodel, obs1.lon, obs1.lat)

hindcasts_det, hindcasts_prob = [], []
i=1
for xtrain, ytrain, xtest, ytest in xc.CrossValidator(model, obs, window=3):
    print("window {}".format(i))
    i += 1
    reg = xc.CCA(search_override=(5,5,3))
#    reg = xc.CCA()
    reg.fit(xtrain, ytrain)
    probs =  reg.predict_proba(xtest)
    preds = reg.predict(xtest)
   
    hindcasts_det.append(preds.isel(time=1))
    hindcasts_prob.append(probs.isel(time=1))
hindcasts_det = xr.concat(hindcasts_det, 'time')
hindcasts_prob = xr.concat(hindcasts_prob, 'time')

ohc = xc.OneHotEncoder() 
ohc.fit(obs)
T = ohc.transform(obs)
clim = xr.ones_like(T) * 0.333

from mprob import mview_probabilistic
fprobs =  reg.predict_proba(fmodel)
mview_probabilistic(fprobs.isel(time=0), title='Precip Forecast, using NMME Predicted Precip ld${ld}, CCA, Valid: ${prd}', savefig='./nmme_predicted_precip_ld${ld}_cca.png')

from rocc import view_mroc
ds = hindcasts_prob.assign_coords(M=(['BN', 'NN', 'AN']))
view_mroc(ds, T, savefig='./nmme_predicted_precip_ld${ld}_cca_roc.png')

groc = xc.GROCS(hindcasts_prob, T)
from pview import view
view(groc, title='NMME, Lead-${ld}, CCA-GROC, Valid: ${prd}', savefig='.//nmme_predicted_precip_ld${ld}_cca_groc.png')


eofPY

python cca_nmme_predicted_precip.py

convert -trim nmme_predicted_precip_ld${ld}_cca.png nmme_predicted_precip_ld${ld}_cca.png

convert -trim nmme_predicted_precip_ld${ld}_cca_roc.png nmme_predicted_precip_ld${ld}_cca_roc.png

convert -trim nmme_predicted_precip_ld${ld}_cca_groc.png nmme_predicted_precip_ld${ld}_cca_groc.png

mv nmme_predicted_precip_ld${ld}_cca.png fig_dir/

mv nmme_predicted_precip_ld${ld}_cca_roc.png fig_dir/

mv nmme_predicted_precip_ld${ld}_cca_groc.png fig_dir/

done



