#!bin/sh

#####################################################################################
# This Shell script and the embeded XCast/python Script are used to                 #
# Download NMME Forecast Data for domain of your interest and                       #
# Plot Season-1 / Season-2 / Season-3 Calibrated Precip Forecsts                    #
# Written/modified by Endalkachew Bekele, NOAA/CPC/International Desks, August 2023 #
# For more details on XCast, please viisit https://xcast-lib.github.io/             #
#####################################################################################

LANG=en_US.utf8
LC_ALL=en_US.utf8
unameout="$(uname -s)"

if [ "$#" -ne 4 ]; then
    echo "***************************************************************************************************************************"
    echo " error while running gen_Ens_nmme_precip_raw.sh"
    echo "----------------- You need to provide 4 arguments -------------------------------------------------------------------------"
    echo "***************************************************************************************************************************"
    echo
    echo "The four arguments reflect geographical extent of your area:"
    echo
    echo "western lon, eastern lon, southern lat and northern lat in degreas, respectively"
    echo
    echo "Use negative sign for longitudes in western, and latitudes in the southern hemisphere"
    echo
    echo "For example, if your area extends from 10W to 12E and from 5S to 10N .."
    echo " you may run the script as .."
    echo
    echo " bash gen_Ens_nmme_precip_raw.sh -10 12 -5 10"
    echo
    echo
exit
fi

wlon1=$1
elon1=$2
slat1=$3
nlat1=$4


wlon2=$1
elon2=$2
slat2=$3
nlat2=$4


mn=`date +"%b"`

for ld in 1 2 3; do

if [ $ld = 1 ]; then

if [ $mn == "Jan" ]; then prd='FMA'; fi
if [ $mn == "Feb" ]; then prd='MAM'; fi
if [ $mn == "Mar" ]; then prd='AMJ'; fi
if [ $mn == "Apr" ]; then prd='MJJ'; fi
if [ $mn == "May" ]; then prd='JJA'; fi
if [ $mn == "Jun" ]; then prd='JAS'; fi
if [ $mn == "Jul" ]; then prd='ASO'; fi
if [ $mn == "Aug" ]; then prd='SON'; fi
if [ $mn == "Sep" ]; then prd='OND'; fi
if [ $mn == "Oct" ]; then prd='NDJ'; fi
if [ $mn == "Nov" ]; then prd='DJF'; fi
if [ $mn == "Dec" ]; then prd='JFM'; fi
fi

if [ $ld = 2 ]; then

if [ $mn == "Jan" ]; then prd='MAM'; fi
if [ $mn == "Feb" ]; then prd='AMJ'; fi
if [ $mn == "Mar" ]; then prd='MJJ'; fi
if [ $mn == "Apr" ]; then prd='JJA'; fi
if [ $mn == "May" ]; then prd='JAS'; fi
if [ $mn == "Jun" ]; then prd='ASO'; fi
if [ $mn == "Jul" ]; then prd='SON'; fi
if [ $mn == "Aug" ]; then prd='OND'; fi
if [ $mn == "Sep" ]; then prd='NDJ'; fi
if [ $mn == "Oct" ]; then prd='DJF'; fi
if [ $mn == "Nov" ]; then prd='JFM'; fi
if [ $mn == "Dec" ]; then prd='FMA'; fi
fi


if [ $ld = 3 ]; then

if [ $mn == "Jan" ]; then prd='AMJ'; fi
if [ $mn == "Feb" ]; then prd='MJJ'; fi
if [ $mn == "Mar" ]; then prd='JJA'; fi
if [ $mn == "Apr" ]; then prd='JAS'; fi
if [ $mn == "May" ]; then prd='ASO'; fi
if [ $mn == "Jun" ]; then prd='SON'; fi
if [ $mn == "Jul" ]; then prd='OND'; fi
if [ $mn == "Aug" ]; then prd='NDJ'; fi
if [ $mn == "Sep" ]; then prd='DJF'; fi
if [ $mn == "Oct" ]; then prd='JFM'; fi
if [ $mn == "Nov" ]; then prd='FMA'; fi
if [ $mn == "Dec" ]; then prd='MAM'; fi
fi

cat>gen_ens.py<<eofPY
#!/usr/bin/env python
# coding: utf-8

import xcast as xc 
import xarray as xr 
import cartopy.crs as ccrs 
import numpy as np
import matplotlib.pyplot as plt 
from mprob import mview_probabilistic
fmodel1 = xr.open_dataset('nmme_ld_${ld}_tercile.nc')
fmodel = fmodel1.precip
fmodel = fmodel.sel(lon=slice(${wlon1}, ${elon1}), lat=slice(${slat1},${nlat1}))
print (fmodel.shape)
mview_probabilistic(fmodel.isel(time=0), title='Precip Forecast, ld${ld}, NNME Raw Ens. Prob., Valid: ${prd}', savefig='./nmme_ld${ld}_ens_raw.png')

eofPY

python gen_ens.py

convert -trim nmme_ld${ld}_ens_raw.png nmme_ld${ld}_ens_raw.png

mv nmme_ld${ld}_ens_raw.png fig_dir/
#srcdir=/cpc/africawrf/ebekele/cca/xcast/subseason/notebooks
#scp $srcdir/nmme_ld${ld}_ens_raw.png ebekele@cpcrzdm:"../ftp/International/ld34/xcast/fig_dir"
#ssh -qX ebekele@rzdm chmod -R o+r ../ftp/International/ld34/xcast/fig_dir


done
