#!bin/sh

#####################################################################################
# This Shell script and the embeded XCast/python Script are used to                 #
# Download NMME Forecast Data for domain of your interest and                       #
# Plot Season-1 / Season-2 / Season-3 Calibrated Precip Forecsts                    #
# Written/modified by Endalkachew Bekele, NOAA/CPC/International Desks, August 2023 #
# For more details on XCast, please viisit https://xcast-lib.github.io/             #
#####################################################################################

LANG=en_US.utf8
LC_ALL=en_US.utf8
unameout="$(uname -s)"

rm *.nc
rm fig_dir/*.png
rm gen_*.py

if [ "$#" -ne 4 ]; then
    echo "***************************************************************************************************************************"
    echo " error while running gen_CCA_nnme_precip_chirps_precip.sh"
    echo "----------------- You need to provide 4 arguments -------------------------------------------------------------------------"
    echo "***************************************************************************************************************************"
    echo
    echo "The four arguments reflect geographical extent of your area:"
    echo
    echo "western lon, eastern lon, southern lat and northern lat in degreas, respectively"
    echo
    echo "Use negative sign for longitudes in western, and latitudes in the southern hemisphere"
    echo
    echo "For example, if your area extends from 10W to 12E and from 5S to 10N .."
    echo " you may run the script as .."
    echo
    echo " bash plot_all.sh -10 12 -5 10"
    echo
    echo
exit
fi

for ld in 1 2 3; do

wget --no-check-certificate https://ftp.cpc.ncep.noaa.gov/International/seasonal/nmme_ld_${ld}_tercile.nc
wget --no-check-certificate https://ftp.cpc.ncep.noaa.gov/International/seasonal/nmme_hind_sst_ld${ld}.nc
wget --no-check-certificate https://ftp.cpc.ncep.noaa.gov/International/seasonal/nmme_hind_precip_ld${ld}.nc
wget --no-check-certificate https://ftp.cpc.ncep.noaa.gov/International/seasonal/chirps_hind_precip_ld${ld}.nc
wget --no-check-certificate https://ftp.cpc.ncep.noaa.gov/International/seasonal/nmme_fcst_sst_ld${ld}.nc
wget --no-check-certificate https://ftp.cpc.ncep.noaa.gov/International/seasonal/nmme_fcst_precip_ld${ld}.nc

done

wlon2=$1
elon2=$2
slat2=$3
nlat2=$4

bash gen_Ens_nmme_precip_raw.sh $1 $2 $3 $4
bash gen_CCA_nmme_predicted_sst.sh $1 $2 $3 $4
bash gen_CCA_nmme_predicted_precip.sh $1 $2 $3 $4
bash gen_ELM_nmme_predicted_precip.sh $1 $2 $3 $4
bash gen_EPOELM_nmme_predicted_precip.sh $1 $2 $3 $4
bash gen_ELR_nmme_predicted_precip.sh $1 $2 $3 $4

rm *.nc
rm *precip*.py


if [ $unameout = "Darwin" ]; then
   /Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome index.html&
   open -a Safari index.html&
else
    firefox index.html&
    google-chrome index.html&
    cygstart index.html
fi

